# -*- coding: utf-8 -*-
"""
Created on Tue Nov 22 03:52:04 2022

@author: richie bao
"""
import geopandas as gpd
import pandas as pd
# from shapely.geometry import shape
# from shapely.affinity import translate
from database import gpd2postSQL,postSQL2gpd
import fiona
fiona.drvsupport.supported_drivers['KML']='rw' # The current version of GeoPandas (0.10.2) can be made to read KML via the following code_snippet

'''
def json2gpd(fp,epsg_original=None,epsg_target=None,relative_x=0,relative_y=0,location=None):    
    df=pd.read_json(fp)
    gdf=gpd.GeoDataFrame.from_features(df["features"])
    if epsg_original:
        gdf.set_crs(epsg_original,inplace=True)
    if epsg_target:
        gdf.to_crs(epsg_target,inplace=True)
        
    if location: # 'TL','TR','BL','BR'
        minx, miny, maxx, maxy=gdf.total_bounds # minx, miny, maxx, maxy        
        if location=="BL":
            xoff=relative_x-minx
            yoff=relative_y-miny
        elif location=="BR":
            xoff=relative_x-maxx
            yoff=relative_y-miny
        elif location=="TL":
            xoff=relative_x-minx
            yoff=relative_y-maxy       
        elif location=="TR":
            xoff=relative_x-maxx
            yoff=relative_y-maxy     
            
        gdf['geometry']=gdf.geometry.apply(lambda row:translate(row,xoff=xoff,yoff=yoff,zoff=0))
        xy_off=[xoff,yoff]            

        return gdf,xy_off
    return gdf
building_gpd,xy_off=json2gpd(json_3DBuilding_fp,4326,32616,0,0,"BL")
'''

def json2gpd(fp,epsg_original=None,epsg_target=None,mask=None):    
    '''
    读取含有地理信息数据的json格式文件为GeoDataFrame格式数据

    Parameters
    ----------
    fp : string
        json文件路径.
    epsg_original : int, optional
        原始投影，epsg编号. The default is None.
    epsg_target : int, optional
        转换为投影，epsg编号. The default is None.
    mask : polygon(Shapely), optional
        提取的范围. The default is None.

    Returns
    -------
    gdf : GeoDataFrame
        转换后的数据.

    '''
    df=pd.read_json(fp)
    gdf=gpd.GeoDataFrame.from_features(df["features"])
    if epsg_original:
        gdf.set_crs(epsg_original,inplace=True)        
    if mask:
        mask_bool=gdf.within(mask)
        gdf=gdf[mask_bool]
    if epsg_target:
        gdf.to_crs(epsg_target,inplace=True)

    return gdf

if __name__=="__main__":
    # A-用于数据提取的边界
    region4chicago_building_fp='../data/region4chicago_building.kml'
    region4chicago_building=gpd.read_file(region4chicago_building_fp,driver='KML')    
    db_args={"myusername":"postgres","mypassword":'123456',"mydatabase":"ghpython","geom_col":"geometry"}
    gpd2postSQL(region4chicago_building,table_name="region4chicago",**db_args)    
    
    # B-读取芝加哥区域json格式建筑地理信息数，并写入数据库和单独存储为GeoJSON地理数据格式
    json_3DBuilding_fp=r"../data/Chicago_3dbuildings.json"   
    building_gpd=json2gpd(json_3DBuilding_fp,4326,32616,region4chicago_building.geometry.values[0]) 
    gpd2postSQL(building_gpd,table_name="chicago_building_mask",**db_args)    
    building_gpd.to_file("../data/chicago_building.geojson",driver='GeoJSON')

    